'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var path = require('path');
var fs = require('fs');

const ts = require('typescript');
const getStyles = (tsFilePath, statements) => {
    const buildSass = (srcFile, sass) => {
        const options = {
            data: fs.readFileSync(srcFile, 'utf8'),
            file: srcFile,
            outputStyle: 'compressed'
        };
        return sass.renderSync(options).css.toString();
    };
    const compileStyle = (statement, cssFullPath) => {
        return statement.moduleSpecifier.getText().includes('.scss')
            ? buildSass(cssFullPath, require('node-sass'))
            : fs.readFileSync(cssFullPath, 'utf8');
    };
    const cssStatements = statements.filter(statement => {
        return ts.isImportDeclaration(statement)
            && (statement.moduleSpecifier.getText().includes('.css')
                || statement.moduleSpecifier.getText().includes('.scss'));
    });
    if (cssStatements) {
        return cssStatements
            .map(statement => {
            const cssRelativePath = statement.moduleSpecifier.getText().replace(/'/g, '');
            const cssFullPath = path.resolve(path.dirname(tsFilePath), cssRelativePath);
            return compileStyle(statement, cssFullPath);
        });
    }
    return [];
};
const createTemplateSpans = (templateSpans, styles) => {
    let lastSpan;
    if (templateSpans.length > 0) {
        lastSpan = templateSpans[templateSpans.length - 1];
    }
    if (lastSpan) {
        const spans = templateSpans.filter(span => {
            return !(span.literal.getText().includes(lastSpan.literal.getText()));
        });
        return [
            ...spans,
            ts.createTemplateSpan(lastSpan.expression, ts.createTemplateTail(lastSpan.literal.text + styles.replace(/\n/gm, '')))
        ];
    }
    return [...templateSpans];
};
const updateRenderMethod = (statement, styles) => {
    let renderMethod = statement.members.find(member => {
        return ts.isMethodDeclaration(member)
            && member.getText().includes('render');
    });
    if (renderMethod) {
        const statement = renderMethod.body.statements[0];
        const template = statement.expression.template;
        const cssStyles = `<style>${styles.join('\n')}</style>`;
        const statements = [
            ts.updateReturn(statement, ts.updateTaggedTemplate(statement, statement.expression.tag, ts.updateTemplateExpression(template, template.head, createTemplateSpans(template.templateSpans, cssStyles))))
        ];
        renderMethod = ts.updateMethod(renderMethod, renderMethod.decorators, renderMethod.modifiers, undefined, renderMethod.name, undefined, undefined, [], undefined, ts.updateBlock(renderMethod.body, [...statements]));
    }
    const members = statement.members.filter(member => {
        return (!(member.getText().includes('render')));
    });
    return [...members, renderMethod];
};
const getStatements = (statements, styles) => {
    return statements.map(statement => {
        if (ts.isClassDeclaration(statement)) {
            if (styles && styles.length > 0) {
                /// @ts-ignore
                statement.members = updateRenderMethod(statement, styles);
            }
        }
        return statement;
    });
};
const inlineStyles = (filePath) => {
    return context => {
        const visitor = (node) => {
            if (Array.isArray(node.statements)) {
                node.statements = getStatements(node.statements, getStyles(filePath, node.statements));
            }
            return ts.visitEachChild(node, (child) => visitor(child), context);
        };
        return visitor;
    };
};

const MagicString = require('magic-string');
const ts$1 = require('typescript');
const resolveId = importee => {
    if (importee.includes('.css') || importee.includes('.scss'))
        return importee;
    return null;
};
const loadById = id => {
    if (id.includes('.css') || id.includes('.scss'))
        return '';
    return null;
};
const transfileModule = (filePath, code) => {
    const { outputText, sourceMapText } = ts$1.transpileModule(code, {
        compilerOptions: {
            module: ts$1.ModuleKind.ES2015,
            target: ts$1.ScriptTarget.ES2018,
            skipLibCheck: true,
            skipDefaultLibCheck: true,
            strictNullChecks: false,
            sourceMap: true
        },
        transformers: {
            before: [
                inlineStyles(filePath)
            ]
        }
    });
    return { code: outputText, map: sourceMapText };
};
function inlineCustomElement() {
    return {
        name: 'inlineCustomElement',
        resolveId: resolveId,
        load: loadById,
        transform(code, id) {
            const magicString = new MagicString(code);
            if (!id.includes(path.join(path.resolve(), 'node_modules'))) {
                return transfileModule(id, magicString.toString());
            }
            return {
                code: magicString.toString(),
                map: magicString.generateMap({ hires: true })
            };
        }
    };
}

exports.inlineCustomElement = inlineCustomElement;
